@@ -12,34 +12,29 @@
 import os
 import re
 import sys
-import time
-import types
 import warnings
 from collections import defaultdict
 from copy import copy
 from os import path
 
-from docutils.frontend import OptionParser
-from docutils.utils import Reporter, get_source_line
-from six import BytesIO, itervalues, class_types, next
+from docutils.utils import get_source_line
+from six import BytesIO, next
 from six.moves import cPickle as pickle
 
-from sphinx import addnodes, versioning
-from sphinx.deprecation import RemovedInSphinx20Warning
+from sphinx import addnodes
+from sphinx.deprecation import RemovedInSphinx20Warning, RemovedInSphinx30Warning
 from sphinx.environment.adapters.indexentries import IndexEntries
 from sphinx.environment.adapters.toctree import TocTree
-from sphinx.errors import SphinxError, ExtensionError
-from sphinx.io import read_doc
+from sphinx.errors import SphinxError, BuildEnvironmentError, DocumentError, ExtensionError
+from sphinx.locale import __
 from sphinx.transforms import SphinxTransformer
-from sphinx.util import get_matching_docs, FilenameUniqDict, status_iterator
-from sphinx.util import logging, rst
-from sphinx.util.console import bold  # type: ignore
-from sphinx.util.docutils import sphinx_domains, WarningStream
+from sphinx.util import get_matching_docs, DownloadFiles, FilenameUniqDict
+from sphinx.util import logging
+from sphinx.util.docutils import LoggingReporter
 from sphinx.util.i18n import find_catalog_files
 from sphinx.util.matching import compile_matchers
 from sphinx.util.nodes import is_translatable
-from sphinx.util.osutil import SEP, ensuredir, relpath
-from sphinx.util.parallel import ParallelTasks, parallel_available, make_chunks
+from sphinx.util.osutil import SEP, relpath
 from sphinx.util.websupport import is_commentable
 
 if False:
@@ -74,6 +69,18 @@ default_settings = {
 # NOTE: increase base version by 2 to have distinct numbers for Py2 and 3
 ENV_VERSION = 54 + (sys.version_info[0] - 2)
 
+# config status
+CONFIG_OK = 1
+CONFIG_NEW = 2
+CONFIG_CHANGED = 3
+CONFIG_EXTENSIONS_CHANGED = 4
+
+CONFIG_CHANGED_REASON = {
+    CONFIG_NEW: __('new config'),
+    CONFIG_CHANGED: __('config changed'),
+    CONFIG_EXTENSIONS_CHANGED: __('extensions changed'),
+}
+
 
 versioning_conditions = {
     'none': False,
@@ -96,81 +103,16 @@ class BuildEnvironment(object):
 
     domains = None  # type: Dict[unicode, Domain]
 
-    # --------- ENVIRONMENT PERSISTENCE ----------------------------------------
-
-    @staticmethod
-    def load(f, app=None):
-        # type: (IO, Sphinx) -> BuildEnvironment
-        try:
-            env = pickle.load(f)
-        except Exception as exc:
-            # This can happen for example when the pickle is from a
-            # different version of Sphinx.
-            raise IOError(exc)
-        if env.version != ENV_VERSION:
-            raise IOError('build environment version not current')
-        if app:
-            env.app = app
-            env.config.values = app.config.values
-            if env.srcdir != app.srcdir:
-                raise IOError('source directory has changed')
-        return env
-
-    @classmethod
-    def loads(cls, string, app=None):
-        # type: (unicode, Sphinx) -> BuildEnvironment
-        io = BytesIO(string)
-        return cls.load(io, app)
-
-    @classmethod
-    def frompickle(cls, filename, app):
-        # type: (unicode, Sphinx) -> BuildEnvironment
-        with open(filename, 'rb') as f:
-            return cls.load(f, app)
-
-    @staticmethod
-    def dump(env, f):
-        # type: (BuildEnvironment, IO) -> None
-        # remove unpicklable attributes
-        app = env.app
-        del env.app
-        values = env.config.values
-        del env.config.values
-        domains = env.domains
-        del env.domains
-        # remove potentially pickling-problematic values from config
-        for key, val in list(vars(env.config).items()):
-            if key.startswith('_') or \
-               isinstance(val, types.ModuleType) or \
-               isinstance(val, types.FunctionType) or \
-               isinstance(val, class_types):
-                del env.config[key]
-        pickle.dump(env, f, pickle.HIGHEST_PROTOCOL)
-        # reset attributes
-        env.domains = domains
-        env.config.values = values
-        env.app = app
-
-    @classmethod
-    def dumps(cls, env):
-        # type: (BuildEnvironment) -> unicode
-        io = BytesIO()
-        cls.dump(env, io)
-        return io.getvalue()
-
-    def topickle(self, filename):
-        # type: (unicode) -> None
-        with open(filename, 'wb') as f:
-            self.dump(self, f)
-
     # --------- ENVIRONMENT INITIALIZATION -------------------------------------
 
-    def __init__(self, app):
+    def __init__(self, app=None):
         # type: (Sphinx) -> None
-        self.app = app
-        self.doctreedir = app.doctreedir
-        self.srcdir = app.srcdir  # type: unicode
-        self.config = app.config  # type: Config
+        self.app = None             # type: Sphinx
+        self.doctreedir = None      # type: unicode
+        self.srcdir = None          # type: unicode
+        self.config = None          # type: Config
+        self.config_status = None   # type: int
+        self.version = None         # type: Dict[unicode, unicode]
 
         # the method of doctree versioning; see set_versioning_method
         self.versioning_condition = None  # type: Union[bool, Callable]
@@ -186,9 +128,6 @@ class BuildEnvironment(object):
         # the function to write warning messages with
         self._warnfunc = None  # type: Callable
 
-        # this is to invalidate old pickles
-        self.version = ENV_VERSION  # type: int
-
         # All "docnames" here are /-separated and relative and exclude
         # the source suffix.
 
@@ -248,13 +187,11 @@ class BuildEnvironment(object):
         self.indexentries = {}      # type: Dict[unicode, List[Tuple[unicode, unicode, unicode, unicode, unicode]]]  # NOQA
                                     # docname -> list of
                                     # (type, unicode, target, aliasname)
-        self.versionchanges = {}    # type: Dict[unicode, List[Tuple[unicode, unicode, int, unicode, unicode, unicode]]]  # NOQA
-                                    # version -> list of (type, docname,
-                                    # lineno, module, descname, content)
 
         # these map absolute path -> (docnames, unique filename)
         self.images = FilenameUniqDict()    # type: FilenameUniqDict
-        self.dlfiles = FilenameUniqDict()   # type: FilenameUniqDict
+        self.dlfiles = DownloadFiles()      # type: DownloadFiles
+                                            # filename -> (set of docnames, destination)
 
         # the original URI for images
         self.original_image_uri = {}  # type: Dict[unicode, unicode]
@@ -266,10 +203,80 @@ class BuildEnvironment(object):
         # attributes of "any" cross references
         self.ref_context = {}       # type: Dict[unicode, Any]
 
+        # set up environment
+        if app:
+            self.setup(app)
+
+    def __getstate__(self):
+        # type: () -> Dict
+        """Obtains serializable data for pickling."""
+        __dict__ = self.__dict__.copy()
+        __dict__.update(app=None, domains={})  # clear unpickable attributes
+        return __dict__
+
+    def __setstate__(self, state):
+        # type: (Dict) -> None
+        self.__dict__.update(state)
+
+    def setup(self, app):
+        # type: (Sphinx) -> None
+        """Set up BuildEnvironment object."""
+        if self.version and self.version != app.registry.get_envversion(app):
+            raise BuildEnvironmentError(__('build environment version not current'))
+        elif self.srcdir and self.srcdir != app.srcdir:
+            raise BuildEnvironmentError(__('source directory has changed'))
+
+        self.app = app
+        self.doctreedir = app.doctreedir
+        self.srcdir = app.srcdir
+        self.version = app.registry.get_envversion(app)
+
+        # initialize domains
+        self.domains = {}
+        for domain in app.registry.create_domains(self):
+            self.domains[domain.name] = domain
+
+        # initialize config
+        self._update_config(app.config)
+
+        # initialie settings
+        self._update_settings(app.config)
+
+    def _update_config(self, config):
+        # type: (Config) -> None
+        """Update configurations by new one."""
+        self.config_status = CONFIG_OK
+        if self.config is None:
+            self.config_status = CONFIG_NEW
+        else:
+            # check if a config value was changed that affects how
+            # doctrees are read
+            for item in config.filter('env'):
+                if self.config[item.name] != item.value:
+                    self.config_status = CONFIG_CHANGED
+                    break
+
+            # this value is not covered by the above loop because it is handled
+            # specially by the config class
+            if self.config.extensions != config.extensions:
+                self.config_status = CONFIG_EXTENSIONS_CHANGED
+
+        self.config = config
+
+    def _update_settings(self, config):
+        # type: (Config) -> None
+        """Update settings by new config."""
+        self.settings['input_encoding'] = config.source_encoding
+        self.settings['trim_footnote_reference_space'] = config.trim_footnote_reference_space
+        self.settings['language_code'] = config.language or 'en'
+
+        # Allow to disable by 3rd party extension (workaround)
+        self.settings.setdefault('smart_quotes', True)
+
     def set_warnfunc(self, func):
         # type: (Callable) -> None
         warnings.warn('env.set_warnfunc() is now deprecated. Use sphinx.util.logging instead.',
-                      RemovedInSphinx20Warning)
+                      RemovedInSphinx20Warning, stacklevel=2)
 
     def set_versioning_method(self, method, compare):
         # type: (unicode, bool) -> None
@@ -284,9 +291,9 @@ class BuildEnvironment(object):
             raise ValueError('invalid versioning method: %r' % method)
         condition = versioning_conditions[method]
         if self.versioning_condition not in (None, condition):
-            raise SphinxError('This environment is incompatible with the '
-                              'selected builder, please choose another '
-                              'doctree directory.')
+            raise SphinxError(__('This environment is incompatible with the '
+                                 'selected builder, please choose another '
+                                 'doctree directory.'))
         self.versioning_condition = condition
         self.versioning_compare = compare
 
@@ -313,10 +320,6 @@ class BuildEnvironment(object):
             self.included.pop(docname, None)
             self.reread_always.discard(docname)
 
-            for version, changes in self.versionchanges.items():
-                new = [change for change in changes if change[1] != docname]
-                changes[:] = new
-
         for domain in self.domains.values():
             domain.clear_doc(docname)
 
@@ -410,37 +413,40 @@ class BuildEnvironment(object):
         """Find all source files in the source dir and put them in
         self.found_docs.
         """
-        matchers = compile_matchers(
-            config.exclude_patterns[:] +
-            config.templates_path +
-            builder.get_asset_paths() +
-            ['**/_sources', '.#*', '**/.#*', '*.lproj/**']
-        )
-        self.found_docs = set()
-        for docname in get_matching_docs(self.srcdir, config.source_suffix,  # type: ignore
-                                         exclude_matchers=matchers):
-            if os.access(self.doc2path(docname), os.R_OK):
-                self.found_docs.add(docname)
-            else:
-                logger.warning("document not readable. Ignored.", location=docname)
+        try:
+            matchers = compile_matchers(
+                config.exclude_patterns[:] +
+                config.templates_path +
+                builder.get_asset_paths() +
+                ['**/_sources', '.#*', '**/.#*', '*.lproj/**']
+            )
+            self.found_docs = set()
+            for docname in get_matching_docs(self.srcdir, config.source_suffix,  # type: ignore
+                                             exclude_matchers=matchers):
+                if os.access(self.doc2path(docname), os.R_OK):
+                    self.found_docs.add(docname)
+                else:
+                    logger.warning(__("document not readable. Ignored."), location=docname)
 
-        # Current implementation is applying translated messages in the reading
-        # phase.Therefore, in order to apply the updated message catalog, it is
-        # necessary to re-process from the reading phase. Here, if dependency
-        # is set for the doc source and the mo file, it is processed again from
-        # the reading phase when mo is updated. In the future, we would like to
-        # move i18n process into the writing phase, and remove these lines.
-        if builder.use_message_catalog:
-            # add catalog mo file dependency
-            for docname in self.found_docs:
-                catalog_files = find_catalog_files(
-                    docname,
-                    self.srcdir,
-                    self.config.locale_dirs,
-                    self.config.language,
-                    self.config.gettext_compact)
-                for filename in catalog_files:
-                    self.dependencies[docname].add(filename)
+            # Current implementation is applying translated messages in the reading
+            # phase.Therefore, in order to apply the updated message catalog, it is
+            # necessary to re-process from the reading phase. Here, if dependency
+            # is set for the doc source and the mo file, it is processed again from
+            # the reading phase when mo is updated. In the future, we would like to
+            # move i18n process into the writing phase, and remove these lines.
+            if builder.use_message_catalog:
+                # add catalog mo file dependency
+                for docname in self.found_docs:
+                    catalog_files = find_catalog_files(
+                        docname,
+                        self.srcdir,
+                        self.config.locale_dirs,
+                        self.config.language,
+                        self.config.gettext_compact)
+                    for filename in catalog_files:
+                        self.dependencies[docname].add(filename)
+        except EnvironmentError as exc:
+            raise DocumentError(__('Failed to scan documents in %s: %r') % (self.srcdir, exc))
 
     def get_outdated_files(self, config_changed):
         # type: (bool) -> Tuple[Set[unicode], Set[unicode], Set[unicode]]
@@ -493,130 +499,6 @@ class BuildEnvironment(object):
 
         return added, changed, removed
 
-    def update(self, config, srcdir, doctreedir):
-        # type: (Config, unicode, unicode) -> List[unicode]
-        """(Re-)read all files new or changed since last update.
-
-        Store all environment docnames in the canonical format (ie using SEP as
-        a separator in place of os.path.sep).
-        """
-        config_changed = False
-        if self.config is None:
-            msg = '[new config] '
-            config_changed = True
-        else:
-            # check if a config value was changed that affects how
-            # doctrees are read
-            for confval in config.filter('env'):
-                if self.config[confval.name] != confval.value:
-                    msg = '[config changed] '
-                    config_changed = True
-                    break
-            else:
-                msg = ''
-            # this value is not covered by the above loop because it is handled
-            # specially by the config class
-            if self.config.extensions != config.extensions:
-                msg = '[extensions changed] '
-                config_changed = True
-        # the source and doctree directories may have been relocated
-        self.srcdir = srcdir
-        self.doctreedir = doctreedir
-        self.find_files(config, self.app.builder)
-        self.config = config
-
-        # this cache also needs to be updated every time
-        self._nitpick_ignore = set(self.config.nitpick_ignore)
-
-        logger.info(bold('updating environment: '), nonl=True)
-
-        added, changed, removed = self.get_outdated_files(config_changed)
-
-        # allow user intervention as well
-        for docs in self.app.emit('env-get-outdated', self, added, changed, removed):
-            changed.update(set(docs) & self.found_docs)
-
-        # if files were added or removed, all documents with globbed toctrees
-        # must be reread
-        if added or removed:
-            # ... but not those that already were removed
-            changed.update(self.glob_toctrees & self.found_docs)
-
-        msg += '%s added, %s changed, %s removed' % (len(added), len(changed),
-                                                     len(removed))
-        logger.info(msg)
-
-        # clear all files no longer present
-        for docname in removed:
-            self.app.emit('env-purge-doc', self, docname)
-            self.clear_doc(docname)
-
-        # read all new and changed files
-        docnames = sorted(added | changed)
-        # allow changing and reordering the list of docs to read
-        self.app.emit('env-before-read-docs', self, docnames)
-
-        # check if we should do parallel or serial read
-        if parallel_available and len(docnames) > 5 and self.app.parallel > 1:
-            par_ok = self.app.is_parallel_allowed('read')
-        else:
-            par_ok = False
-
-        if par_ok:
-            self._read_parallel(docnames, self.app, nproc=self.app.parallel)
-        else:
-            self._read_serial(docnames, self.app)
-
-        if config.master_doc not in self.all_docs:
-            raise SphinxError('master file %s not found' %
-                              self.doc2path(config.master_doc))
-
-        for retval in self.app.emit('env-updated', self):
-            if retval is not None:
-                docnames.extend(retval)
-
-        return sorted(docnames)
-
-    def _read_serial(self, docnames, app):
-        # type: (List[unicode], Sphinx) -> None
-        for docname in status_iterator(docnames, 'reading sources... ', "purple",
-                                       len(docnames), self.app.verbosity):
-            # remove all inventory entries for that file
-            app.emit('env-purge-doc', self, docname)
-            self.clear_doc(docname)
-            self.read_doc(docname, app)
-
-    def _read_parallel(self, docnames, app, nproc):
-        # type: (List[unicode], Sphinx, int) -> None
-        # clear all outdated docs at once
-        for docname in docnames:
-            app.emit('env-purge-doc', self, docname)
-            self.clear_doc(docname)
-
-        def read_process(docs):
-            # type: (List[unicode]) -> unicode
-            self.app = app
-            for docname in docs:
-                self.read_doc(docname, app)
-            # allow pickling self to send it back
-            return BuildEnvironment.dumps(self)
-
-        def merge(docs, otherenv):
-            # type: (List[unicode], unicode) -> None
-            env = BuildEnvironment.loads(otherenv)
-            self.merge_info_from(docs, env, app)
-
-        tasks = ParallelTasks(nproc)
-        chunks = make_chunks(docnames, nproc)
-
-        for chunk in status_iterator(chunks, 'reading sources... ', "purple",
-                                     len(chunks), self.app.verbosity):
-            tasks.add_task(read_process, chunk, merge)
-
-        # make sure all threads have finished
-        logger.info(bold('waiting for workers...'))
-        tasks.join()
-
     def check_dependents(self, app, already):
         # type: (Sphinx, Set[unicode]) -> Iterator[unicode]
         to_rewrite = []  # type: List[unicode]
@@ -637,55 +519,6 @@ class BuildEnvironment(object):
         self.temp_data['default_domain'] = \
             self.domains.get(self.config.primary_domain)
 
-        self.settings['input_encoding'] = self.config.source_encoding
-        self.settings['trim_footnote_reference_space'] = \
-            self.config.trim_footnote_reference_space
-        self.settings['gettext_compact'] = self.config.gettext_compact
-
-        self.settings['language_code'] = self.config.language or 'en'
-
-        # Allow to disable by 3rd party extension (workaround)
-        self.settings.setdefault('smart_quotes', True)
-
-    def read_doc(self, docname, app=None):
-        # type: (unicode, Sphinx) -> None
-        """Parse a file and add/update inventory entries for the doctree."""
-        self.prepare_settings(docname)
-
-        docutilsconf = path.join(self.srcdir, 'docutils.conf')
-        # read docutils.conf from source dir, not from current dir
-        OptionParser.standard_config_files[1] = docutilsconf
-        if path.isfile(docutilsconf):
-            self.note_dependency(docutilsconf)
-
-        with sphinx_domains(self), rst.default_role(docname, self.config.default_role):
-            doctree = read_doc(self.app, self, self.doc2path(docname))
-
-        # post-processing
-        for domain in itervalues(self.domains):
-            domain.process_doc(self, docname, doctree)
-
-        # allow extension-specific post-processing
-        if app:
-            app.emit('doctree-read', doctree)
-
-        # store time of reading, for outdated files detection
-        # (Some filesystems have coarse timestamp resolution;
-        # therefore time.time() can be older than filesystem's timestamp.
-        # For example, FAT32 has 2sec timestamp resolution.)
-        self.all_docs[docname] = max(
-            time.time(), path.getmtime(self.doc2path(docname)))
-
-        if self.versioning_condition:
-            # add uids for versioning
-            versioning.prepare(doctree)
-
-        # cleanup
-        self.temp_data.clear()
-        self.ref_context.clear()
-
-        self.write_doctree(docname, doctree)
-
     # utilities to use while reading a document
 
     @property
@@ -732,13 +565,6 @@ class BuildEnvironment(object):
         """
         self.reread_always.add(self.docname)
 
-    def note_versionchange(self, type, version, node, lineno):
-        # type: (unicode, unicode, nodes.Node, int) -> None
-        self.versionchanges.setdefault(version, []).append(
-            (type, self.temp_data['docname'], lineno,
-             self.ref_context.get('py:module'),
-             self.temp_data.get('object'), node.astext()))
-
     def note_toctree(self, docname, toctreenode):
         # type: (unicode, addnodes.toctree) -> None
         """Note a TOC tree directive in a document and gather information about
@@ -746,7 +572,7 @@ class BuildEnvironment(object):
         """
         warnings.warn('env.note_toctree() is deprecated. '
                       'Use sphinx.environment.adapters.toctree.TocTree instead.',
-                      RemovedInSphinx20Warning)
+                      RemovedInSphinx20Warning, stacklevel=2)
         TocTree(self).note(docname, toctreenode)
 
     def get_toc_for(self, docname, builder):
@@ -754,7 +580,7 @@ class BuildEnvironment(object):
         """Return a TOC nodetree -- for use on the same page only!"""
         warnings.warn('env.get_toc_for() is deprecated. '
                       'Use sphinx.environment.adapters.toctre.TocTree instead.',
-                      RemovedInSphinx20Warning)
+                      RemovedInSphinx20Warning, stacklevel=2)
         return TocTree(self).get_toc_for(docname, builder)
 
     def get_toctree_for(self, docname, builder, collapse, **kwds):
@@ -762,7 +588,7 @@ class BuildEnvironment(object):
         """Return the global TOC nodetree."""
         warnings.warn('env.get_toctree_for() is deprecated. '
                       'Use sphinx.environment.adapters.toctre.TocTree instead.',
-                      RemovedInSphinx20Warning)
+                      RemovedInSphinx20Warning, stacklevel=2)
         return TocTree(self).get_toctree_for(docname, builder, collapse, **kwds)
 
     def get_domain(self, domainname):
@@ -774,7 +600,7 @@ class BuildEnvironment(object):
         try:
             return self.domains[domainname]
         except KeyError:
-            raise ExtensionError('Domain %r is not registered' % domainname)
+            raise ExtensionError(__('Domain %r is not registered') % domainname)
 
     # --------- RESOLVING REFERENCES AND TOCTREES ------------------------------
 
@@ -785,24 +611,9 @@ class BuildEnvironment(object):
         with open(doctree_filename, 'rb') as f:
             doctree = pickle.load(f)
         doctree.settings.env = self
-        doctree.reporter = Reporter(self.doc2path(docname), 2, 5, stream=WarningStream())
+        doctree.reporter = LoggingReporter(self.doc2path(docname))
         return doctree
 
-    def write_doctree(self, docname, doctree):
-        # type: (unicode, nodes.Node) -> None
-        """Write the doctree to a file."""
-        # make it picklable
-        doctree.reporter = None
-        doctree.transformer = None
-        doctree.settings.warning_stream = None
-        doctree.settings.env = None
-        doctree.settings.record_dependencies = None
-
-        doctree_filename = self.doc2path(docname, self.doctreedir, '.doctree')
-        ensuredir(path.dirname(doctree_filename))
-        with open(doctree_filename, 'wb') as f:
-            pickle.dump(doctree, f, pickle.HIGHEST_PROTOCOL)
-
     def get_and_resolve_doctree(self, docname, builder, doctree=None,
                                 prune_toctrees=True, includehidden=False):
         # type: (unicode, Builder, nodes.Node, bool, bool) -> nodes.Node
@@ -872,7 +683,7 @@ class BuildEnvironment(object):
         # type: (Builder, bool, Pattern) -> List[Tuple[unicode, List[Tuple[unicode, List[unicode]]]]]  # NOQA
         warnings.warn('env.create_index() is deprecated. '
                       'Use sphinx.environment.adapters.indexentreis.IndexEntries instead.',
-                      RemovedInSphinx20Warning)
+                      RemovedInSphinx20Warning, stacklevel=2)
         return IndexEntries(self).create_index(builder,
                                                group_entries=group_entries,
                                                _fixre=_fixre)
@@ -884,7 +695,7 @@ class BuildEnvironment(object):
         def traverse_toctree(parent, docname):
             # type: (unicode, unicode) -> Iterator[Tuple[unicode, unicode]]
             if parent == docname:
-                logger.warning('self referenced toctree found. Ignored.', location=docname)
+                logger.warning(__('self referenced toctree found. Ignored.'), location=docname)
                 return
 
             # traverse toctree by pre-order
@@ -925,10 +736,130 @@ class BuildEnvironment(object):
                     continue
                 if 'orphan' in self.metadata[docname]:
                     continue
-                logger.warning('document isn\'t included in any toctree',
+                logger.warning(__('document isn\'t included in any toctree'),
                                location=docname)
 
         # call check-consistency for all extensions
         for domain in self.domains.values():
             domain.check_consistency()
         self.app.emit('env-check-consistency', self)
+
+    # --------- METHODS FOR COMPATIBILITY --------------------------------------
+
+    def update(self, config, srcdir, doctreedir):
+        # type: (Config, unicode, unicode) -> List[unicode]
+        warnings.warn('env.update() is deprecated. Please use builder.read() instead.',
+                      RemovedInSphinx30Warning, stacklevel=2)
+        return self.app.builder.read()
+
+    def _read_serial(self, docnames, app):
+        # type: (List[unicode], Sphinx) -> None
+        warnings.warn('env._read_serial() is deprecated. Please use builder.read() instead.',
+                      RemovedInSphinx30Warning, stacklevel=2)
+        return self.app.builder._read_serial(docnames)
+
+    def _read_parallel(self, docnames, app, nproc):
+        # type: (List[unicode], Sphinx, int) -> None
+        warnings.warn('env._read_parallel() is deprecated. Please use builder.read() instead.',
+                      RemovedInSphinx30Warning, stacklevel=2)
+        return self.app.builder._read_parallel(docnames, nproc)
+
+    def read_doc(self, docname, app=None):
+        # type: (unicode, Sphinx) -> None
+        warnings.warn('env.read_doc() is deprecated. Please use builder.read_doc() instead.',
+                      RemovedInSphinx30Warning, stacklevel=2)
+        self.app.builder.read_doc(docname)
+
+    def write_doctree(self, docname, doctree):
+        # type: (unicode, nodes.Node) -> None
+        warnings.warn('env.write_doctree() is deprecated. '
+                      'Please use builder.write_doctree() instead.',
+                      RemovedInSphinx30Warning, stacklevel=2)
+        self.app.builder.write_doctree(docname, doctree)
+
+    @property
+    def _nitpick_ignore(self):
+        # type: () -> List[unicode]
+        warnings.warn('env._nitpick_ignore is deprecated. '
+                      'Please use config.nitpick_ignore instead.',
+                      RemovedInSphinx30Warning, stacklevel=2)
+        return self.config.nitpick_ignore
+
+    @staticmethod
+    def load(f, app=None):
+        # type: (IO, Sphinx) -> BuildEnvironment
+        warnings.warn('BuildEnvironment.load() is deprecated. '
+                      'Please use pickle.load() instead.',
+                      RemovedInSphinx30Warning, stacklevel=2)
+        try:
+            env = pickle.load(f)
+        except Exception as exc:
+            # This can happen for example when the pickle is from a
+            # different version of Sphinx.
+            raise IOError(exc)
+        if app:
+            env.app = app
+            env.config.values = app.config.values
+        return env
+
+    @classmethod
+    def loads(cls, string, app=None):
+        # type: (unicode, Sphinx) -> BuildEnvironment
+        warnings.warn('BuildEnvironment.loads() is deprecated. '
+                      'Please use pickle.loads() instead.',
+                      RemovedInSphinx30Warning, stacklevel=2)
+        io = BytesIO(string)
+        return cls.load(io, app)
+
+    @classmethod
+    def frompickle(cls, filename, app):
+        # type: (unicode, Sphinx) -> BuildEnvironment
+        warnings.warn('BuildEnvironment.frompickle() is deprecated. '
+                      'Please use pickle.load() instead.',
+                      RemovedInSphinx30Warning, stacklevel=2)
+        with open(filename, 'rb') as f:
+            return cls.load(f, app)
+
+    @staticmethod
+    def dump(env, f):
+        # type: (BuildEnvironment, IO) -> None
+        warnings.warn('BuildEnvironment.dump() is deprecated. '
+                      'Please use pickle.dump() instead.',
+                      RemovedInSphinx30Warning, stacklevel=2)
+        pickle.dump(env, f, pickle.HIGHEST_PROTOCOL)
+
+    @classmethod
+    def dumps(cls, env):
+        # type: (BuildEnvironment) -> unicode
+        warnings.warn('BuildEnvironment.dumps() is deprecated. '
+                      'Please use pickle.dumps() instead.',
+                      RemovedInSphinx30Warning, stacklevel=2)
+        io = BytesIO()
+        cls.dump(env, io)
+        return io.getvalue()
+
+    def topickle(self, filename):
+        # type: (unicode) -> None
+        warnings.warn('env.topickle() is deprecated. '
+                      'Please use pickle.dump() instead.',
+                      RemovedInSphinx30Warning, stacklevel=2)
+        with open(filename, 'wb') as f:
+            self.dump(self, f)
+
+    @property
+    def versionchanges(self):
+        # type: () -> Dict[unicode, List[Tuple[unicode, unicode, int, unicode, unicode, unicode]]]  # NOQA
+        warnings.warn('env.versionchanges() is deprecated. '
+                      'Please use ChangeSetDomain instead.',
+                      RemovedInSphinx30Warning, stacklevel=2)
+        return self.domaindata['changeset']['changes']
+
+    def note_versionchange(self, type, version, node, lineno):
+        # type: (unicode, unicode, nodes.Node, int) -> None
+        warnings.warn('env.note_versionchange() is deprecated. '
+                      'Please use ChangeSetDomain.note_changeset() instead.',
+                      RemovedInSphinx30Warning, stacklevel=2)
+        node['type'] = type
+        node['version'] = version
+        node.line = lineno
+        self.get_domain('changeset').note_changeset(node)  # type: ignore
