@@ -65,7 +65,7 @@ var Stemmer = function() {
 }
 """                             # type: unicode
 
-    _word_re = re.compile(r'\w+(?u)')
+    _word_re = re.compile(r'(?u)\w+')
 
     def __init__(self, options):
         # type: (Dict) -> None
@@ -85,7 +85,7 @@ var Stemmer = function() {
         at white spaces, which should be enough for most languages except CJK
         languages.
         """
-        return self._word_re.findall(input)  # type: ignore
+        return self._word_re.findall(input)
 
     def stem(self, word):
         # type: (unicode) -> unicode
@@ -331,13 +331,13 @@ class IndexBuilder(object):
         for domainname, domain in sorted(iteritems(self.env.domains)):
             for fullname, dispname, type, docname, anchor, prio in \
                     sorted(domain.get_objects()):
-                # XXX use dispname?
                 if docname not in fn2index:
                     continue
                 if prio < 0:
                     continue
                 fullname = htmlescape(fullname)
-                prefix, name = rpartition(fullname, '.')
+                dispname = htmlescape(dispname)
+                prefix, name = rpartition(dispname, '.')
                 pdict = rv.setdefault(prefix, {})
                 try:
                     typeindex = otypes[domainname, type]
