@@ -71,6 +71,7 @@ def compile_latex_document(app):
             copyfile('SphinxTests.tex',
                      app.config.latex_engine + '/SphinxTests.tex')
             p = Popen([app.config.latex_engine,
+                       '--halt-on-error',
                        '--interaction=nonstopmode',
                        '-output-directory=%s' % app.config.latex_engine,
                        'SphinxTests.tex'],
@@ -467,7 +468,7 @@ def test_babel_with_language_ru(app, status, warning):
     assert '\\addto\\captionsrussian{\\renewcommand{\\tablename}{Table.}}\n' in result
     assert (u'\\addto\\extrasrussian{\\def\\pageautorefname'
             u'{\u0441\u0442\u0440\u0430\u043d\u0438\u0446\u0430}}\n' in result)
-    assert '\\shorthandoff' not in result
+    assert '\\shorthandoff{"}' in result
 
 
 @pytest.mark.sphinx(
@@ -529,7 +530,7 @@ def test_babel_with_unknown_language(app, status, warning):
     assert '\\addto\\captionsenglish{\\renewcommand{\\figurename}{Fig.}}\n' in result
     assert '\\addto\\captionsenglish{\\renewcommand{\\tablename}{Table.}}\n' in result
     assert '\\addto\\extrasenglish{\\def\\pageautorefname{page}}\n' in result
-    assert '\\shorthandoff' not in result
+    assert '\\shorthandoff' in result
 
     assert "WARNING: no Babel option known for language 'unknown'" in warning.getvalue()
 
@@ -590,15 +591,8 @@ def test_footnote(app, status, warning):
     assert ('\\begin{footnote}[2]\\sphinxAtStartFootnote\nauto numbered\n%\n'
             '\\end{footnote}') in result
     assert '\\begin{footnote}[3]\\sphinxAtStartFootnote\nnamed\n%\n\\end{footnote}' in result
-    assert '{\\hyperref[\\detokenize{footnote:bar}]{\\sphinxcrossref{{[}bar{]}}}}' in result
-    assert ('\\bibitem[bar]{\\detokenize{bar}}'
-            '{\\phantomsection\\label{\\detokenize{footnote:bar}} ') in result
-    assert ('\\bibitem[bar]{\\detokenize{bar}}'
-            '{\\phantomsection\\label{\\detokenize{footnote:bar}} '
-            '\ncite') in result
-    assert ('\\bibitem[bar]{\\detokenize{bar}}'
-            '{\\phantomsection\\label{\\detokenize{footnote:bar}} '
-            '\ncite\n}') in result
+    assert '\\sphinxcite{footnote:bar}' in result
+    assert ('\\bibitem[bar]{footnote:bar}\ncite\n') in result
     assert '\\sphinxcaption{Table caption \\sphinxfootnotemark[4]' in result
     assert ('\\hline%\n\\begin{footnotetext}[4]\\sphinxAtStartFootnote\n'
             'footnote in table caption\n%\n\\end{footnotetext}\\ignorespaces %\n'
@@ -620,34 +614,32 @@ def test_reference_in_caption_and_codeblock_in_footnote(app, status, warning):
     print(status.getvalue())
     print(warning.getvalue())
     assert ('\\caption{This is the figure caption with a reference to '
-            '\\label{\\detokenize{index:id2}}'
-            '{\\hyperref[\\detokenize{index:authoryear}]'
-            '{\\sphinxcrossref{{[}AuthorYear{]}}}}.}' in result)
+            '\\sphinxcite{index:authoryear}.}' in result)
     assert '\\chapter{The section with a reference to {[}AuthorYear{]}}' in result
     assert ('\\sphinxcaption{The table title with a reference'
             ' to {[}AuthorYear{]}}' in result)
-    assert '\\paragraph{The rubric title with a reference to {[}AuthorYear{]}}' in result
-    assert ('\\chapter{The section with a reference to \\sphinxfootnotemark[4]}\n'
+    assert '\\subsubsection*{The rubric title with a reference to {[}AuthorYear{]}}' in result
+    assert ('\\chapter{The section with a reference to \\sphinxfootnotemark[5]}\n'
             '\\label{\\detokenize{index:the-section-with-a-reference-to}}'
-            '%\n\\begin{footnotetext}[4]\\sphinxAtStartFootnote\n'
+            '%\n\\begin{footnotetext}[5]\\sphinxAtStartFootnote\n'
             'Footnote in section\n%\n\\end{footnotetext}') in result
     assert ('\\caption{This is the figure caption with a footnote to '
-            '\\sphinxfootnotemark[6].}\\label{\\detokenize{index:id27}}\\end{figure}\n'
-            '%\n\\begin{footnotetext}[6]\\sphinxAtStartFootnote\n'
+            '\\sphinxfootnotemark[7].}\\label{\\detokenize{index:id29}}\\end{figure}\n'
+            '%\n\\begin{footnotetext}[7]\\sphinxAtStartFootnote\n'
             'Footnote in caption\n%\n\\end{footnotetext}')in result
-    assert ('\\sphinxcaption{footnote \\sphinxfootnotemark[7] in '
-            'caption of normal table}\\label{\\detokenize{index:id28}}') in result
-    assert ('\\caption{footnote \\sphinxfootnotemark[8] '
-            'in caption \\sphinxfootnotemark[9] of longtable\\strut}') in result
-    assert ('\\endlastfoot\n%\n\\begin{footnotetext}[8]\\sphinxAtStartFootnote\n'
+    assert ('\\sphinxcaption{footnote \\sphinxfootnotemark[8] in '
+            'caption of normal table}\\label{\\detokenize{index:id30}}') in result
+    assert ('\\caption{footnote \\sphinxfootnotemark[9] '
+            'in caption \\sphinxfootnotemark[10] of longtable\\strut}') in result
+    assert ('\\endlastfoot\n%\n\\begin{footnotetext}[9]\\sphinxAtStartFootnote\n'
             'Foot note in longtable\n%\n\\end{footnotetext}\\ignorespaces %\n'
-            '\\begin{footnotetext}[9]\\sphinxAtStartFootnote\n'
+            '\\begin{footnotetext}[10]\\sphinxAtStartFootnote\n'
             'Second footnote in caption of longtable\n') in result
     assert ('This is a reference to the code-block in the footnote:\n'
             '{\\hyperref[\\detokenize{index:codeblockinfootnote}]'
             '{\\sphinxcrossref{\\DUrole{std,std-ref}{I am in a footnote}}}}') in result
     assert ('&\nThis is one more footnote with some code in it %\n'
-            '\\begin{footnote}[10]\\sphinxAtStartFootnote\n'
+            '\\begin{footnote}[11]\\sphinxAtStartFootnote\n'
             'Third footnote in longtable\n') in result
     assert ('\\end{sphinxVerbatim}\n%\n\\end{footnote}.\n') in result
     assert '\\begin{sphinxVerbatim}[commandchars=\\\\\\{\\}]' in result
@@ -666,14 +658,12 @@ def test_latex_show_urls_is_inline(app, status, warning):
             'footnote in bar\n%\n\\end{footnote} in bar.rst') in result
     assert ('Auto footnote number %\n\\begin{footnote}[1]\\sphinxAtStartFootnote\n'
             'footnote in baz\n%\n\\end{footnote} in baz.rst') in result
-    assert ('\\phantomsection\\label{\\detokenize{index:id30}}'
+    assert ('\\phantomsection\\label{\\detokenize{index:id32}}'
             '{\\hyperref[\\detokenize{index:the-section'
             '-with-a-reference-to-authoryear}]'
             '{\\sphinxcrossref{The section with a reference to '
-            '\\phantomsection\\label{\\detokenize{index:id1}}'
-            '{\\hyperref[\\detokenize{index:authoryear}]'
-            '{\\sphinxcrossref{{[}AuthorYear{]}}}}}}}') in result
-    assert ('\\phantomsection\\label{\\detokenize{index:id31}}'
+            '\\sphinxcite{index:authoryear}}}}') in result
+    assert ('\\phantomsection\\label{\\detokenize{index:id33}}'
             '{\\hyperref[\\detokenize{index:the-section-with-a-reference-to}]'
             '{\\sphinxcrossref{The section with a reference to }}}' in result)
     assert ('First footnote: %\n\\begin{footnote}[2]\\sphinxAtStartFootnote\n'
@@ -682,13 +672,15 @@ def test_latex_show_urls_is_inline(app, status, warning):
             'Second\n%\n\\end{footnote}') in result
     assert '\\sphinxhref{http://sphinx-doc.org/}{Sphinx} (http://sphinx-doc.org/)' in result
     assert ('Third footnote: %\n\\begin{footnote}[3]\\sphinxAtStartFootnote\n'
-            'Third\n%\n\\end{footnote}') in result
+            'Third \\sphinxfootnotemark[4]\n%\n\\end{footnote}%\n'
+            '\\begin{footnotetext}[4]\\sphinxAtStartFootnote\n'
+            'Footnote inside footnote\n%\n\\end{footnotetext}\\ignorespaces') in result
     assert ('\\sphinxhref{http://sphinx-doc.org/~test/}{URL including tilde} '
             '(http://sphinx-doc.org/\\textasciitilde{}test/)') in result
     assert ('\\item[{\\sphinxhref{http://sphinx-doc.org/}{URL in term} '
             '(http://sphinx-doc.org/)}] \\leavevmode\nDescription' in result)
-    assert ('\\item[{Footnote in term \\sphinxfootnotemark[5]}] '
-            '\\leavevmode%\n\\begin{footnotetext}[5]\\sphinxAtStartFootnote\n'
+    assert ('\\item[{Footnote in term \\sphinxfootnotemark[6]}] '
+            '\\leavevmode%\n\\begin{footnotetext}[6]\\sphinxAtStartFootnote\n'
             'Footnote in term\n%\n\\end{footnotetext}\\ignorespaces \n'
             'Description') in result
     assert ('\\item[{\\sphinxhref{http://sphinx-doc.org/}{Term in deflist} '
@@ -711,13 +703,11 @@ def test_latex_show_urls_is_footnote(app, status, warning):
             'footnote in bar\n%\n\\end{footnote} in bar.rst') in result
     assert ('Auto footnote number %\n\\begin{footnote}[2]\\sphinxAtStartFootnote\n'
             'footnote in baz\n%\n\\end{footnote} in baz.rst') in result
-    assert ('\\phantomsection\\label{\\detokenize{index:id30}}'
+    assert ('\\phantomsection\\label{\\detokenize{index:id32}}'
             '{\\hyperref[\\detokenize{index:the-section-with-a-reference-to-authoryear}]'
             '{\\sphinxcrossref{The section with a reference '
-            'to \\phantomsection\\label{\\detokenize{index:id1}}'
-            '{\\hyperref[\\detokenize{index:authoryear}]'
-            '{\\sphinxcrossref{{[}AuthorYear{]}}}}}}}') in result
-    assert ('\\phantomsection\\label{\\detokenize{index:id31}}'
+            'to \\sphinxcite{index:authoryear}}}}') in result
+    assert ('\\phantomsection\\label{\\detokenize{index:id33}}'
             '{\\hyperref[\\detokenize{index:the-section-with-a-reference-to}]'
             '{\\sphinxcrossref{The section with a reference to }}}') in result
     assert ('First footnote: %\n\\begin{footnote}[3]\\sphinxAtStartFootnote\n'
@@ -728,22 +718,25 @@ def test_latex_show_urls_is_footnote(app, status, warning):
             '%\n\\begin{footnote}[4]\\sphinxAtStartFootnote\n'
             '\\sphinxnolinkurl{http://sphinx-doc.org/}\n%\n\\end{footnote}') in result
     assert ('Third footnote: %\n\\begin{footnote}[6]\\sphinxAtStartFootnote\n'
-            'Third\n%\n\\end{footnote}') in result
+            'Third \\sphinxfootnotemark[7]\n%\n\\end{footnote}%\n'
+            '\\begin{footnotetext}[7]\\sphinxAtStartFootnote\n'
+            'Footnote inside footnote\n%\n'
+            '\\end{footnotetext}\\ignorespaces') in result
     assert ('\\sphinxhref{http://sphinx-doc.org/~test/}{URL including tilde}'
             '%\n\\begin{footnote}[5]\\sphinxAtStartFootnote\n'
             '\\sphinxnolinkurl{http://sphinx-doc.org/~test/}\n%\n\\end{footnote}') in result
     assert ('\\item[{\\sphinxhref{http://sphinx-doc.org/}'
-            '{URL in term}\\sphinxfootnotemark[8]}] '
-            '\\leavevmode%\n\\begin{footnotetext}[8]\\sphinxAtStartFootnote\n'
+            '{URL in term}\\sphinxfootnotemark[9]}] '
+            '\\leavevmode%\n\\begin{footnotetext}[9]\\sphinxAtStartFootnote\n'
             '\\sphinxnolinkurl{http://sphinx-doc.org/}\n%\n'
             '\\end{footnotetext}\\ignorespaces \nDescription') in result
-    assert ('\\item[{Footnote in term \\sphinxfootnotemark[10]}] '
-            '\\leavevmode%\n\\begin{footnotetext}[10]\\sphinxAtStartFootnote\n'
+    assert ('\\item[{Footnote in term \\sphinxfootnotemark[11]}] '
+            '\\leavevmode%\n\\begin{footnotetext}[11]\\sphinxAtStartFootnote\n'
             'Footnote in term\n%\n\\end{footnotetext}\\ignorespaces \n'
             'Description') in result
     assert ('\\item[{\\sphinxhref{http://sphinx-doc.org/}{Term in deflist}'
-            '\\sphinxfootnotemark[9]}] '
-            '\\leavevmode%\n\\begin{footnotetext}[9]\\sphinxAtStartFootnote\n'
+            '\\sphinxfootnotemark[10]}] '
+            '\\leavevmode%\n\\begin{footnotetext}[10]\\sphinxAtStartFootnote\n'
             '\\sphinxnolinkurl{http://sphinx-doc.org/}\n%\n'
             '\\end{footnotetext}\\ignorespaces \nDescription') in result
     assert ('\\sphinxurl{https://github.com/sphinx-doc/sphinx}\n' in result)
@@ -764,13 +757,11 @@ def test_latex_show_urls_is_no(app, status, warning):
             'footnote in bar\n%\n\\end{footnote} in bar.rst') in result
     assert ('Auto footnote number %\n\\begin{footnote}[1]\\sphinxAtStartFootnote\n'
             'footnote in baz\n%\n\\end{footnote} in baz.rst') in result
-    assert ('\\phantomsection\\label{\\detokenize{index:id30}}'
+    assert ('\\phantomsection\\label{\\detokenize{index:id32}}'
             '{\\hyperref[\\detokenize{index:the-section-with-a-reference-to-authoryear}]'
             '{\\sphinxcrossref{The section with a reference '
-            'to \\phantomsection\\label{\\detokenize{index:id1}}'
-            '{\\hyperref[\\detokenize{index:authoryear}]'
-            '{\\sphinxcrossref{{[}AuthorYear{]}}}}}}}') in result
-    assert ('\\phantomsection\\label{\\detokenize{index:id31}}'
+            'to \\sphinxcite{index:authoryear}}}}') in result
+    assert ('\\phantomsection\\label{\\detokenize{index:id33}}'
             '{\\hyperref[\\detokenize{index:the-section-with-a-reference-to}]'
             '{\\sphinxcrossref{The section with a reference to }}}' in result)
     assert ('First footnote: %\n\\begin{footnote}[2]\\sphinxAtStartFootnote\n'
@@ -779,12 +770,14 @@ def test_latex_show_urls_is_no(app, status, warning):
             'Second\n%\n\\end{footnote}') in result
     assert '\\sphinxhref{http://sphinx-doc.org/}{Sphinx}' in result
     assert ('Third footnote: %\n\\begin{footnote}[3]\\sphinxAtStartFootnote\n'
-            'Third\n%\n\\end{footnote}') in result
+            'Third \\sphinxfootnotemark[4]\n%\n\\end{footnote}%\n'
+            '\\begin{footnotetext}[4]\\sphinxAtStartFootnote\n'
+            'Footnote inside footnote\n%\n\\end{footnotetext}\\ignorespaces') in result
     assert '\\sphinxhref{http://sphinx-doc.org/~test/}{URL including tilde}' in result
     assert ('\\item[{\\sphinxhref{http://sphinx-doc.org/}{URL in term}}] '
             '\\leavevmode\nDescription') in result
-    assert ('\\item[{Footnote in term \\sphinxfootnotemark[5]}] '
-            '\\leavevmode%\n\\begin{footnotetext}[5]\\sphinxAtStartFootnote\n'
+    assert ('\\item[{Footnote in term \\sphinxfootnotemark[6]}] '
+            '\\leavevmode%\n\\begin{footnotetext}[6]\\sphinxAtStartFootnote\n'
             'Footnote in term\n%\n\\end{footnotetext}\\ignorespaces \n'
             'Description') in result
     assert ('\\item[{\\sphinxhref{http://sphinx-doc.org/}{Term in deflist}}] '
@@ -1207,9 +1200,13 @@ def test_latex_index(app, status, warning):
     app.builder.build_all()
 
     result = (app.outdir / 'Python.tex').text(encoding='utf8')
-    assert 'A \\index{famous}famous \\index{equation}equation:\n' in result
-    assert '\n\\index{Einstein}\\index{relativity}\\ignorespaces \nand' in result
-    assert '\n\\index{main \\sphinxleftcurlybrace}\\ignorespaces ' in result
+    assert ('A \\index{famous@\\spxentry{famous}}famous '
+            '\\index{equation@\\spxentry{equation}}equation:\n' in result)
+    assert ('\n\\index{Einstein@\\spxentry{Einstein}}'
+            '\\index{relativity@\\spxentry{relativity}}'
+            '\\ignorespaces \nand') in result
+    assert ('\n\\index{main \\sphinxleftcurlybrace{}@\\spxentry{'
+            'main \\sphinxleftcurlybrace{}}}\\ignorespaces ' in result)
 
 
 @pytest.mark.sphinx('latex', testroot='latex-equations')
@@ -1237,11 +1234,39 @@ def test_latex_nested_enumerated_list(app, status, warning):
     app.builder.build_all()
 
     result = (app.outdir / 'test.tex').text(encoding='utf8')
-    assert r'\setcounter{enumi}{4}' in result
-    assert r'\setcounter{enumii}{3}' in result
-    assert r'\setcounter{enumiii}{9}' in result
-    assert r'\setcounter{enumiv}{23}' in result
-    assert r'\setcounter{enumii}{2}' in result
+    assert ('\\def\\theenumi{\\arabic{enumi}}\n'
+            '\\def\\labelenumi{\\theenumi .}\n'
+            '\\makeatletter\\def\\p@enumii{\\p@enumi \\theenumi .}\\makeatother\n'
+            '\\setcounter{enumi}{4}\n' in result)
+    assert ('\\def\\theenumii{\\alph{enumii}}\n'
+            '\\def\\labelenumii{\\theenumii .}\n'
+            '\\makeatletter\\def\\p@enumiii{\\p@enumii \\theenumii .}\\makeatother\n'
+            '\\setcounter{enumii}{3}\n' in result)
+    assert ('\\def\\theenumiii{\\arabic{enumiii}}\n'
+            '\\def\\labelenumiii{\\theenumiii )}\n'
+            '\\makeatletter\\def\\p@enumiv{\\p@enumiii \\theenumiii )}\\makeatother\n'
+            '\\setcounter{enumiii}{9}\n' in result)
+    assert ('\\def\\theenumiv{\\arabic{enumiv}}\n'
+            '\\def\\labelenumiv{(\\theenumiv )}\n'
+            '\\makeatletter\\def\\p@enumv{\\p@enumiv (\\theenumiv )}\\makeatother\n'
+            '\\setcounter{enumiv}{23}\n' in result)
+    assert ('\\def\\theenumii{\\roman{enumii}}\n'
+            '\\def\\labelenumii{\\theenumii .}\n'
+            '\\makeatletter\\def\\p@enumiii{\\p@enumii \\theenumii .}\\makeatother\n'
+            '\\setcounter{enumii}{2}\n' in result)
+
+
+@pytest.mark.sphinx('latex', testroot='footnotes')
+def test_latex_thebibliography(app, status, warning):
+    app.builder.build_all()
+
+    result = (app.outdir / 'Python.tex').text(encoding='utf8')
+    print(result)
+    assert ('\\begin{sphinxthebibliography}{AuthorYe}\n'
+            '\\bibitem[AuthorYear]{index:authoryear}\n'
+            'Author, Title, Year\n'
+            '\\end{sphinxthebibliography}\n' in result)
+    assert '\\sphinxcite{index:authoryear}' in result
 
 
 @pytest.mark.sphinx('latex', testroot='glossary')
@@ -1249,20 +1274,22 @@ def test_latex_glossary(app, status, warning):
     app.builder.build_all()
 
     result = (app.outdir / 'test.tex').text(encoding='utf8')
-    assert (u'\\item[{änhlich\\index{änhlich|textbf}\\phantomsection'
+    assert (u'\\item[{änhlich\\index{änhlich@\\spxentry{änhlich}|spxpagem}'
+            r'\phantomsection'
             r'\label{\detokenize{index:term-anhlich}}}] \leavevmode' in result)
-    assert (r'\item[{boson\index{boson|textbf}\phantomsection'
+    assert (r'\item[{boson\index{boson@\spxentry{boson}|spxpagem}\phantomsection'
             r'\label{\detokenize{index:term-boson}}}] \leavevmode' in result)
-    assert (r'\item[{\sphinxstyleemphasis{fermion}\index{fermion|textbf}'
+    assert (r'\item[{\sphinxstyleemphasis{fermion}'
+            r'\index{fermion@\spxentry{fermion}|spxpagem}'
             r'\phantomsection'
             r'\label{\detokenize{index:term-fermion}}}] \leavevmode' in result)
-    assert (r'\item[{tauon\index{tauon|textbf}\phantomsection'
+    assert (r'\item[{tauon\index{tauon@\spxentry{tauon}|spxpagem}\phantomsection'
             r'\label{\detokenize{index:term-tauon}}}] \leavevmode'
-            r'\item[{myon\index{myon|textbf}\phantomsection'
+            r'\item[{myon\index{myon@\spxentry{myon}|spxpagem}\phantomsection'
             r'\label{\detokenize{index:term-myon}}}] \leavevmode'
-            r'\item[{electron\index{electron|textbf}\phantomsection'
+            r'\item[{electron\index{electron@\spxentry{electron}|spxpagem}\phantomsection'
             r'\label{\detokenize{index:term-electron}}}] \leavevmode' in result)
-    assert (u'\\item[{über\\index{über|textbf}\\phantomsection'
+    assert (u'\\item[{über\\index{über@\\spxentry{über}|spxpagem}\\phantomsection'
             r'\label{\detokenize{index:term-uber}}}] \leavevmode' in result)
 
 
@@ -1279,7 +1306,8 @@ def test_latex_labels(app, status, warning):
             r'\label{\detokenize{index:figure1}}'
             r'\end{figure}' in result)
     assert (r'\caption{labeled figure}'
-            r'\label{\detokenize{index:figure3}}'
+            '\\label{\\detokenize{index:figure3}}\n'
+            '\\begin{sphinxlegend}\nwith a legend\n\\end{sphinxlegend}\n'
             r'\end{figure}' in result)
 
     # code-blocks
