@@ -9,14 +9,15 @@
     :license: BSD, see LICENSE for details.
 """
 
-import pytest
 import os
 
+import pytest
+
 from sphinx.config import Config
 from sphinx.directives.code import LiteralIncludeReader
 from sphinx.testing.util import etree_parse
 
-DUMMY_CONFIG = Config(None, None, {}, '')
+DUMMY_CONFIG = Config({}, {})
 
 
 @pytest.fixture(scope='module')
@@ -355,6 +356,8 @@ def test_code_block_emphasize_latex(app, status, warning):
     latex = (app.outdir / 'Python.tex').text(encoding='utf-8').replace('\r\n', '\n')
     includes = '\\fvset{hllines={, 5, 6, 13, 14, 15, 24, 25, 26, 27,}}%\n'
     assert includes in latex
+    includes = '\\end{sphinxVerbatim}\n\\sphinxresetverbatimhllines\n'
+    assert includes in latex
 
 
 @pytest.mark.sphinx('xml', testroot='directive-code')
