@@ -131,8 +131,8 @@ def test_text_emit_warnings(app, warning):
     app.build()
     # test warnings in translation
     warnings = getwarning(warning)
-    warning_expr = u'.*/warnings.txt:4: ' \
-                   u'WARNING: Inline literal start-string without end-string.\n'
+    warning_expr = ('.*/warnings.txt:4:<translated>:1: '
+                   'WARNING: Inline literal start-string without end-string.\n')
     assert_re_search(warning_expr, warnings)
 
 
@@ -588,7 +588,8 @@ def test_gettext_buildr_ignores_only_directive(app):
 def test_gettext_dont_rebuild_mo(make_app, app_params, build_mo):
     # --- don't rebuild by .mo mtime
     def get_number_of_update_targets(app_):
-        updated = app_.env.update(app_.config, app_.srcdir, app_.doctreedir)
+        app_.env.find_files(app_.config, app_.builder)
+        _, updated, _ = app_.env.get_outdated_files(config_changed=False)
         return len(updated)
 
     args, kwargs = app_params
@@ -769,12 +770,14 @@ def test_html_rebuild_mo(app):
     app.build()
     # --- rebuild by .mo mtime
     app.builder.build_update()
-    updated = app.env.update(app.config, app.srcdir, app.doctreedir)
+    app.env.find_files(app.config, app.builder)
+    _, updated, _ = app.env.get_outdated_files(config_changed=False)
     assert len(updated) == 0
 
     mtime = (app.srcdir / 'xx' / 'LC_MESSAGES' / 'bom.mo').stat().st_mtime
     (app.srcdir / 'xx' / 'LC_MESSAGES' / 'bom.mo').utime((mtime + 5, mtime + 5))
-    updated = app.env.update(app.config, app.srcdir, app.doctreedir)
+    app.env.find_files(app.config, app.builder)
+    _, updated, _ = app.env.get_outdated_files(config_changed=False)
     assert len(updated) == 1
 
 
@@ -791,9 +794,13 @@ def test_xml_footnotes(app, warning):
     assert_elem(
         para0[0],
         ['I18N WITH FOOTNOTE', 'INCLUDE THIS CONTENTS',
-         '2', '[ref]', '1', '100', '*', '.'],
+         '2', '[ref]', '1', '100', '*', '. SECOND FOOTNOTE_REF', '100', '.'],
         ['i18n-with-footnote', 'ref'])
 
+    # check node_id for footnote_references which refer same footnote (refs: #3002)
+    assert para0[0][4].text == para0[0][6].text == '100'
+    assert para0[0][4].attrib['ids'] != para0[0][6].attrib['ids']
+
     footnote0 = secs[0].findall('footnote')
     assert_elem(
         footnote0[0],
@@ -845,8 +852,8 @@ def test_xml_footnote_backlinks(app):
     footnote0 = secs[0].findall('footnote')
     for footnote in footnote0:
         ids = footnote.attrib.get('ids')
-        backrefs = footnote.attrib.get('backrefs')
-        assert refid2id[ids] == backrefs
+        backrefs = footnote.attrib.get('backrefs').split()
+        assert refid2id[ids] in backrefs
 
 
 @sphinx_intl
