@@ -232,7 +232,7 @@ def test_Signature_partialmethod():
                     reason='type annotation test is available on py34 or above')
 def test_Signature_annotations():
     from typing_test_data import (
-        f0, f1, f2, f3, f4, f5, f6, f7, f8, f9, f10, f11, f12, f13, f14, Node)
+        f0, f1, f2, f3, f4, f5, f6, f7, f8, f9, f10, f11, f12, f13, f14, f15, Node)
 
     # Class annotations
     sig = inspect.Signature(f0).format_args()
@@ -297,6 +297,10 @@ def test_Signature_annotations():
     sig = inspect.Signature(f14).format_args()
     assert sig == '() -> Any'
 
+    # ForwardRef
+    sig = inspect.Signature(f15).format_args()
+    assert sig == '(x: Unknown, y: int) -> Any'
+
     # type hints by string
     sig = inspect.Signature(Node.children).format_args()
     if (3, 5, 0) <= sys.version_info < (3, 5, 3):
@@ -373,6 +377,24 @@ def test_dictionary_sorting():
     assert description == "{'a': 1, 'b': 4, 'c': 3, 'd': 2}"
 
 
+def test_set_sorting():
+    set_ = set("gfedcba")
+    description = inspect.object_description(set_)
+    if PY3:
+        assert description == "{'a', 'b', 'c', 'd', 'e', 'f', 'g'}"
+    else:
+        assert description == "set(['a', 'b', 'c', 'd', 'e', 'f', 'g'])"
+
+
+def test_set_sorting_fallback():
+    set_ = set((None, 1))
+    description = inspect.object_description(set_)
+    if PY3:
+        assert description in ("{1, None}", "{None, 1}")
+    else:
+        assert description in ("set([1, None])", "set([None, 1])")
+
+
 def test_dict_customtype():
     class CustomType(object):
         def __init__(self, value):
