@@ -11,19 +11,18 @@
 from __future__ import print_function
 
 import codecs
+import os
+
+import pytest
 from docutils import nodes
 
 from sphinx.errors import SphinxWarning
+from sphinx.testing.util import strip_escseq
 from sphinx.util import logging
 from sphinx.util.console import colorize
 from sphinx.util.logging import is_suppressed_warning
 from sphinx.util.parallel import ParallelTasks
 
-import pytest
-from sphinx.testing.util import strip_escseq
-
-import os
-
 
 def test_info_and_warning(app, status, warning):
     app.verbosity = 2
@@ -172,6 +171,37 @@ def test_warningiserror(app, status, warning):
         logger.warning('%s')
 
 
+def test_info_location(app, status, warning):
+    logging.setup(app, status, warning)
+    logger = logging.getLogger(__name__)
+
+    logger.info('message1', location='index')
+    assert 'index.txt: message1' in status.getvalue()
+
+    logger.info('message2', location=('index', 10))
+    assert 'index.txt:10: message2' in status.getvalue()
+
+    logger.info('message3', location=None)
+    assert '\nmessage3' in status.getvalue()
+
+    node = nodes.Node()
+    node.source, node.line = ('index.txt', 10)
+    logger.info('message4', location=node)
+    assert 'index.txt:10: message4' in status.getvalue()
+
+    node.source, node.line = ('index.txt', None)
+    logger.info('message5', location=node)
+    assert 'index.txt:: message5' in status.getvalue()
+
+    node.source, node.line = (None, 10)
+    logger.info('message6', location=node)
+    assert '<unknown>:10: message6' in status.getvalue()
+
+    node.source, node.line = (None, None)
+    logger.info('message7', location=node)
+    assert '\nmessage7' in status.getvalue()
+
+
 def test_warning_location(app, status, warning):
     logging.setup(app, status, warning)
     logger = logging.getLogger(__name__)
@@ -183,7 +213,7 @@ def test_warning_location(app, status, warning):
     assert 'index.txt:10: WARNING: message2' in warning.getvalue()
 
     logger.warning('message3', location=None)
-    assert colorize('darkred', 'WARNING: message3') in warning.getvalue()
+    assert colorize('red', 'WARNING: message3') in warning.getvalue()
 
     node = nodes.Node()
     node.source, node.line = ('index.txt', 10)
@@ -200,7 +230,7 @@ def test_warning_location(app, status, warning):
 
     node.source, node.line = (None, None)
     logger.warning('message7', location=node)
-    assert colorize('darkred', 'WARNING: message7') in warning.getvalue()
+    assert colorize('red', 'WARNING: message7') in warning.getvalue()
 
 
 def test_pending_warnings(app, status, warning):
@@ -236,7 +266,7 @@ def test_colored_logs(app, status, warning):
     assert colorize('darkgray', 'message1') in status.getvalue()
     assert 'message2\n' in status.getvalue()  # not colored
     assert 'message3\n' in status.getvalue()  # not colored
-    assert colorize('darkred', 'WARNING: message4') in warning.getvalue()
+    assert colorize('red', 'WARNING: message4') in warning.getvalue()
     assert 'WARNING: message5\n' in warning.getvalue()  # not colored
     assert colorize('darkred', 'WARNING: message6') in warning.getvalue()
 
